#include "usart.h"

//------------------------------------------------------------------------------
volatile BOOL usart_ready;
volatile U08 usart_buffer[BUFFER_SIZE];

static U08 buffer_index;
static U08 receive_char;
static BOOL receive_suspend;

//------------------------------------------------------------------------------
void usart_init(void) 
{
  buffer_index = 0;
  usart_ready = FALSE;
  receive_suspend = FALSE;

  UBRR0H = 0x00; // Baudraten Register MSB setzen
  UBRR0L = 0x0C; // Baudraten Register LSB setzen (38400 Baud 16MHz)
  UCSR0A = 0x00;
  UCSR0B = (1<<RXCIE0) | (1<<RXEN0) | (1<<TXEN0); // Sender + Empfnger ein, RX Interrupt
  UCSR0C = (1<<UCSZ01) | (1<<UCSZ00); // 8-Bit Zeichenlnge
}

//------------------------------------------------------------------------------
void usart_write_char(U08 data)
{
  while (!(UCSR0A & (1<<UDRE0))) ; // Warten bis Sendepuffer leer ist
  UDR0 = data;
}

//------------------------------------------------------------------------------
void usart_writeln_flash_str(const U08 *str)
{

  while (*str)
  {
    usart_write_char(*str++);
  }
  usart_write_char(CR);
  usart_write_char(LF);
}

//------------------------------------------------------------------------------
void usart_write_str(pU08 str)
{

  while (*str)
  {
    usart_write_char(*str++);
  }
}

//------------------------------------------------------------------------------
void usart_writeln_str(pU08 str)
{
  usart_write_str(str);
  usart_write_char(CR);
  usart_write_char(LF);
}

//------------------------------------------------------------------------------
void usart_write_U08(U08 data) // Byte ausgeben
{
  usart_write_str(U08_to_str(data));
}

//------------------------------------------------------------------------------
void usart_write_U16(U16 data) // Byte ausgeben
{
  usart_write_str(U16_to_str(data));
}

//------------------------------------------------------------------------------
void usart_write_U08_hex(U08 data) // Byte hexadezimal ausgeben
{
  usart_write_str(U08_to_hex(data));
}

//------------------------------------------------------------------------------
SIGNAL (SIG_USART_RECV)
{
  receive_char = toupper(UDR0);
  if (usart_ready) // Wenn Ready-Flag gesetzt ist, dann Empfang ignorieren
  {
    return;
  }

  if (receive_char == CR) // Wenn Zeichen CR ist, dann Ready-Flag setzen
  {
    usart_buffer[buffer_index] = 0; // Stringterminierung
    buffer_index = 0;
    usart_ready = TRUE;
    receive_suspend = FALSE;
    return;    
  }

  if (receive_char == '/' || receive_char == ';' || receive_char == '_')
  {                      
    receive_suspend = TRUE;
  }  

  if (receive_suspend)
  {
    return;
  }

  if (buffer_index < BUFFER_SIZE - 1) // Empfangszeichen speichern
  {
    usart_buffer[buffer_index++] = receive_char;    
  }
return;
}


